package  bigal
{
	import com.adobe.utils.AGALMiniAssembler;
	import flash.display.BitmapData;
	import flash.display.BlendMode;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DTextureFormat;
	import flash.display3D.Context3DTriangleFace;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.textures.Texture;
	import flash.display3D.VertexBuffer3D;
	import flash.events.Event;
	import flash.geom.Matrix3D;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class AgalBase extends Sprite 
	{
		private var context3DID:int;
		static public var context:Context3D;
		static public var assembler:AGALMiniAssembler;
		
		static private var _viewportWidth:int;
		static private var _viewportHeight:int;
		static private var _antialias:int = 0;
		static private var _rect:Rectangle = new Rectangle();
		
		
        static public var backgroundColor:int = 0x101010;
		static public var camera:Camera;
		
		private var transform3D:Matrix3D;
		private var invertTransform3D:Matrix3D;
		
		private var models:Vector.<Model>;
		public var world:Object3D;
		
		static private var _debug:Boolean = true;
		private var _text:TextField;
		
		public function AgalBase( context3DID:int = 0, width:int = NaN, height:int = NaN ) 
		{
			
			this.context3DID = context3DID;
			
			_viewportWidth = width;
			_viewportHeight = height;
			stage.stage3Ds[ this.context3DID ].addEventListener( Event.CONTEXT3D_CREATE, onContext3DCreate );
			stage.stage3Ds[ this.context3DID ].requestContext3D();
			
		}
		
		private function onContext3DCreate(e:Event):void 
		{
			
			stage.scaleMode = StageScaleMode.NO_SCALE;
            stage.align = StageAlign.TOP_LEFT;
			
		 
			context = stage.stage3Ds[ context3DID ].context3D;
			context.enableErrorChecking = debug;
			assembler = new AGALMiniAssembler();
            context.setCulling( Context3DTriangleFace.NONE );
			
			
			if( isNaN( _viewportWidth )  ) _viewportWidth = stage.stageWidth;
			if( isNaN( _viewportHeight )  ) _viewportHeight = stage.stageHeight;
			
			world = new Object3D();
			transform3D = new Matrix3D();
			models = new Vector.<Model>();
			
			camera = new Camera();
			camera.fov = 45;
			
			
			resetBackBuffer();
			stage.addEventListener( Event.RESIZE, onResize );
			
			init();
			
		}
		
		private function onResize(e:Event):void 
		{
			
			_viewportWidth = stage.stageWidth;
			_viewportHeight = stage.stageHeight;
			resetBackBuffer();
			
		}
		
		//called when context is ready, overridden by subclasses
		public function init():void {}
		
		public function render():void
		{
			
			context.clear( ( backgroundColor >> 16 & 0xFF ) / 0xFF, ( backgroundColor >> 8 & 0xFF ) / 0xFF, ( backgroundColor & 0xFF ) / 0xFF, 0 ); 
			
			for each( var model:Model in models )
			{
				if ( model.visible )
				{
					transform3D.identity();
					
					transform3D.append( model.transform );
					transform3D.append( world.transform );
					transform3D.append( camera.transform );
					transform3D.append( camera.projection );
					setVertexMatrixConstant( 0, transform3D, true );//	vc 0
					
					model.render( context );
				}
			}
			
			context.present();
			
		}
		
		/**
		 * registers a model to the models' pool
		 * @param	model a Model to register
		 */
		public function register( model:Model ):void
		{
			models.push( model );
		}
		
		/**
		 * unregisters a model from the models' pool
		 * @param	model a Model to unregister
		 */
		public function unregister( model:Model ):void
		{
			models.splice( models.indexOf( model ), 1 );
		}
		
		
		
		//SETTING VERTEX CONSTANT : VC#
		static public function setVertexVectorConstant( id:int, constant:Vector.<Number> ):void
		{
			
			context.setProgramConstantsFromVector( Context3DProgramType.VERTEX, id, constant );
			
		}
		
		//SETTING VERTEX CONSTANT : VC#
		static public function setVertexMatrixConstant( id:int, matrix:Matrix3D, transpose:Boolean = false ):void
		{
			
			context.setProgramConstantsFromMatrix( Context3DProgramType.VERTEX, id, matrix, transpose );
            
		}
		
		//SETTING FRAGMENT CONSTANT : FC#
		static public function setFragmentVectorConstant( id:int, constant:Vector.<Number> ):void
		{
			
			context.setProgramConstantsFromVector( Context3DProgramType.FRAGMENT, id, constant );
			
		}
		
		//SETTING FRAGMENT CONSTANT : FC#
		static public function setFragmentMatrixConstant( id:int, matrix:Matrix3D, transpose:Boolean = false ):void
		{
			
			context.setProgramConstantsFromMatrix( Context3DProgramType.FRAGMENT, id, matrix, transpose );
            
		}
		
		//CREATE A VERTEX BUFFER
		static public function getVertexBuffer( vData:Vector.<Number>, dataPerVertex:int = 3 ):VertexBuffer3D
		{
			
			var vertexBuffer:VertexBuffer3D = context.createVertexBuffer( vData.length / dataPerVertex, dataPerVertex );
			vertexBuffer.uploadFromVector( vData, 0, vData.length / dataPerVertex );
			return vertexBuffer;
			
		}
		
		//SETTING A VERTEX BUFFER : VA#
		static public function setVertexBufferAt( id:int, vertexBuffer:VertexBuffer3D, offset:int = 0, format:String = Context3DVertexBufferFormat.FLOAT_3 ):VertexBuffer3D
		{
			
			context.setVertexBufferAt( id, vertexBuffer, offset, format );
			return vertexBuffer;
			
		}
		
		//CREATE A TEXTURE OBJECT
		static public function getTexture( bd:BitmapData, format:String = Context3DTextureFormat.BGRA, optimizeForRTT:Boolean = false ):Texture
		{
			
			var texture:Texture = context.createTexture( bd.width, bd.height, Context3DTextureFormat.BGRA, false);
			updateTexture( texture, bd );
			return texture;
			
		}
		
		//UPLOADS A BITMAPDATA TO A TEXTURE
		static public function updateTexture( texture:Texture, bd:BitmapData ):Texture
		{
			
			texture.uploadFromBitmapData( bd );
			return texture;
			
		}
		
		//SETTING A TEXTURE : FS#
		static public function setTextureAt( id:int, texture:Texture ):Texture
		{
			
			context.setTextureAt( id, texture );
			return texture;
			
		}
		
		
		
		//CREATE AN INDEX BUFFER, USED TO DRAWTRIANGLES
		static public function getIndexBuffer( iData:Vector.<uint> ):IndexBuffer3D
		{
			
			var indexBuffer:IndexBuffer3D = context.createIndexBuffer( iData.length  );
			indexBuffer.uploadFromVector( iData, 0, iData.length );
			return indexBuffer;
			
		}
		
		
		public function get debug():Boolean 
		{
			return _debug;
		}
		
		public function set debug(value:Boolean):void 
		{
			_debug = value;
			if( context ) context.enableErrorChecking = value;
		}
		
		public function get viewportWidth():int 
		{
			return _viewportWidth;
		}
		
		public function set viewportWidth(value:int):void 
		{
			_viewportWidth = value;
			resetBackBuffer();
		}
		
		public function get viewportHeight():int 
		{
			return _viewportHeight;
		}
		
		public function set viewportHeight(value:int):void 
		{
			_viewportHeight = value;
			resetBackBuffer();
		}
		
		public function get antialias():int
		{
			return _antialias;
		}
		
		public function set antialias(value:int):void 
		{
			_antialias = value;
			resetBackBuffer();
		}
		
		private function resetBackBuffer():void
		{
			context.configureBackBuffer( _viewportWidth, _viewportHeight, _antialias, true );
			_rect.width = _viewportWidth;
			_rect.height = _viewportHeight;
			camera.aspectRatio = viewportWidth / viewportHeight;
		}
		
		static public function get rect():Rectangle 
		{
			return _rect;
		}
		
		public function get text():TextField 
		{
			if ( _text == null )
			{
				
				_text = new TextField();
				_text.defaultTextFormat = new TextFormat( "Verdana", 10, 0, true, false, false );
				_text.blendMode = BlendMode.INVERT;
				_text.autoSize = TextFieldAutoSize.LEFT;
				addChild( _text );
				
			}
			return _text;
		}
		
		
	}
}