package bigal 
{
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class Camera extends Object3D
	{
		private var X_AXIS:Vector3D = Vector3D.X_AXIS.clone();
		private var Y_AXIS:Vector3D = Vector3D.Y_AXIS.clone();
		private var Z_AXIS:Vector3D = Vector3D.Z_AXIS.clone();
		
		private var _target:Vector3D;
		private var lens:Lens;
		
		public function Camera( position:Vector3D = null, lens:Lens = null ) 
		{
			
			position ||= new Vector3D();
			this.x = position.x;
			this.y = position.y;
			this.z = position.z;
			this.lens = lens || new Lens();
			
			X_AXIS.negate();
			Y_AXIS.negate();
			Z_AXIS.negate();
			
		}
		
		override public function get transform():Matrix3D 
		{
			
			if ( geomDirty )
			{
				
				super.transform;
				
				if ( _target != null )	_transform.pointAt( _target, Z_AXIS, Y_AXIS );
				_transform.invert();
				geomDirty = false;
				
			}
			
			return _transform;
			
		}
		
		public function lookat( target:Vector3D ):void
		{
			_target = target;
			geomDirty = true;
		}
		
		public function get direction():Vector3D
		{
			return transform.transformVector( Vector3D.Z_AXIS );
		}
		
		/**
		 * LENS PROPERTIES
		 */
		
		public function get fov():Number { return lens.fov; }
		public function set fov( value:Number ):void{ lens.fov = value; geomDirty = true; }
		
		public function get aspectRatio():Number { return lens.aspectRatio; }
		public function set aspectRatio( value:Number ):void { lens.aspectRatio = value; geomDirty = true; }
		
		public function get near():Number { return lens.near; }
		public function set near( value:Number ):void{ lens.near = value; geomDirty = true; }
		
		public function get far():Number { return lens.far; }
		public function set far( value:Number ):void{ lens.far = value; geomDirty = true; }
		
		public function get projection():Matrix3D{ return lens; }
		
		
	}

}