package bigal 
{
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class Object3D 
	{
		static public const RADIANS:String = "radians";
		static public const DEGREES:String = "degrees";
		
		static private var DEG_TO_RAD:Number = Math.PI / 180;
		static private var RAD_TO_DEG:Number = 180 / Math.PI;
		
		protected var _transform:Matrix3D = new Matrix3D();		
		protected var _position:Vector3D = new Vector3D( 0,0,0 );
		protected var _rotation:Vector3D = new Vector3D( 0,0,0 );
		protected var _scale:Vector3D = new Vector3D( 1,1,1 );
		protected var geomDirty:Boolean = true;
		
		
		public function Object3D() {}
		
		/**************************************
		
			TRANSFORMATIONS
		
		**************************************/
		
		public function get transform():Matrix3D 
		{
			if ( geomDirty )
			{
				_transform.recompose( Vector.<Vector3D>([ _position, _rotation, _scale ]) );
				geomDirty = false;
			}
			return _transform;
		}
		
		public function set transform(value:Matrix3D):void 
		{
			_transform = value;
			geomDirty = false;
		}
		
		
		public function get position():Vector3D {	return _position;	}
		public function set position(value:Vector3D):void 
		{
			_position = value;
			geomDirty = true;
		}
		
		public function get rotation():Vector3D {	return _rotation;	}
		public function set rotation(value:Vector3D):void 
		{
			_rotation = value;
			geomDirty = true;
		}
		
		
		public function set x(value:Number):void {		position.x = value;		geomDirty = true;		}
		public function set y(value:Number):void {		position.y = value;		geomDirty = true;		}
		public function set z(value:Number):void {		position.z = value;		geomDirty = true;		}
		
		public function get x():Number {		return position.x;		}
		public function get y():Number {		return position.y;		}
		public function get z():Number {		return position.z;		}
		
		public function set rotationX(value:Number):void {		rotation.x = value * DEG_TO_RAD;		geomDirty = true;		}
		public function set rotationY(value:Number):void {		rotation.y = value * DEG_TO_RAD;		geomDirty = true;		}
		public function set rotationZ(value:Number):void {		rotation.z = value * DEG_TO_RAD;		geomDirty = true;		}
		
		public function get rotationX():Number {		return rotation.x * RAD_TO_DEG;		}
		public function get rotationY():Number {		return rotation.y * RAD_TO_DEG;		}
		public function get rotationZ():Number {		return rotation.z * RAD_TO_DEG;		}
		
		public function set scale(value:Number):void {		_scale.x = _scale.y = _scale.z = value;		geomDirty = true;		}
		public function set scaleX(value:Number):void {		_scale.x = value;		geomDirty = true;		}
		public function set scaleY(value:Number):void {		_scale.y = value;		geomDirty = true;		}
		public function set scaleZ(value:Number):void {		_scale.z = value;		geomDirty = true;		}
		
		public function get scaleX():Number {		return _scale.x;		}
		public function get scaleY():Number {		return _scale.y;		}
		public function get scaleZ():Number {		return _scale.z;		}
		
		
		static public function set ANGLE_TYPE( value:String ):void
		{
			DEG_TO_RAD = value == DEGREES ? ( Math.PI / 180 ) : 1;
			RAD_TO_DEG = value == DEGREES ? ( 180 / Math.PI ) : 1;
		}
		
	}

}